

#include "audioplayer.h"
#include "midiparser.h"
#include "mididrv.h"


#include <f32file.h>

extern MidiParser *MidiParser_createRO();
extern MidiParser *MidiParser_createEUP();
extern MidiDriver *MidiDriver_ADLIB_create(SoundMixer *mixer);


enum
    {
    ENone, ERo, ESo, EXmidi, ESmf 
    };

LOCAL_C TInt RegocMidi(const TDesC8& aData)
    {
    const TInt KHheadersz = 52;
    if(aData.Length() > KHheadersz)
        {
        if(aData.Mid(4, 3).Compare(_L8("ROL")) == 0)
            return ERo;
        if(aData.Mid(8, 2).Compare(_L8("SO")) == 0)
            return ERo;
        for(TInt i = 0; i < KHheadersz - 4; i++)
            {
            TPtrC8 h = aData.Mid(i, 4);
            if(h.Compare(_L8("MThd")) == 0)
                return ESmf;
            if(h.Compare(_L8("FORM")) == 0)
                return EXmidi;
            }
        }
    return ENone;
    }

LOCAL_C MidiParser* CreateMidiParser(const TDesC8& aData)
    {
    switch(RegocMidi(aData))
        {
        case ERo:
            return MidiParser_createRO();
        case ESo:
            return MidiParser_createEUP();
        case EXmidi:
            return MidiParser::createParser_XMIDI();
        case ESmf:
             return MidiParser::createParser_SMF();
        default:
            return NULL;
        }
    }

CMidiPlayer::CMidiPlayer(SoundMixer& aMixer) : iMixer(aMixer) 
    {
    }

void CMidiPlayer::ConstructL()
    {
    iDriver = MidiDriver_ADLIB_create(&iMixer);

    const TInt err = iDriver->open();
    
    if(err != 0)
        {
        TBuf<64> errs;
        errs.Copy(TPtrC8((const TUint8*)iDriver->getErrorName(err)).Left(errs.MaxLength()));
        User::InfoPrint(errs);
        User::Leave(err);
        }
    }

void CMidiPlayer::PlayMidiFromFileL(const TDesC& aFileName, TBool aLooping)
    {
    TEntry entry;
	RFs fs;
	fs.Connect();
    User::LeaveIfError(fs.Entry(aFileName, entry));
    HBufC8* buf = HBufC8::NewLC(entry.iSize);

    RFile file;
    User::LeaveIfError(file.Open(fs, aFileName, EFileRead));
    TPtr8 ptr = buf->Des();
    file.Read(ptr);
    file.Close();
	fs.Close();
    CleanupStack::Pop(); //buf
    PlayMidiL(buf, aLooping);
    }

void CMidiPlayer::PlayMidiL(HBufC8* aData, TBool aLooping)
    {
    Stop();
    delete iMidiDataBuffer;
    iMidiDataBuffer = aData;
    PlayMidiL(*iMidiDataBuffer, EFalse, aLooping);
    }

TBool CMidiPlayer::IsPlaying() const
    { 
    if(iParser != NULL)
        return iParser->getTick() > 0;
    return EFalse;
    }

TBool CMidiPlayer::IsLoaded() const
    {
	return iParser != NULL;
    }

void CMidiPlayer::PlayMidiL(const TDesC8& aMidiData, TBool aCopy, TBool aLooping)
    {
    if(aCopy)
        {
        delete iMidiDataBuffer;
        iMidiData = NULL;
        iMidiDataBuffer = aMidiData.AllocL();
        iMidiData = iMidiDataBuffer;
        }
    else
        {
        iMidiData = &aMidiData;
        }

    Stop();   

    iParser = CreateMidiParser(*iMidiData);

    __ASSERT_ALWAYS(iParser != NULL, User::Leave(KErrNotFound));

    iParser->setMidiDriver(iDriver);

    byte* data = const_cast<byte*>(iMidiData->Ptr());

    iParser->loadMusic(data, iMidiData->Size());

    iParser->setTimerRate((iDriver->getBaseTempo() * 128) >> 7);
    iParser->property(MidiParser::mpAutoLoop, aLooping);
    iDriver->setTimerCallback(iParser, MidiParser::timerCallback);
    }

void CMidiPlayer::SetVolume(TInt aVolume)
    {
	iVolume = aVolume;
    }

void CMidiPlayer::SetPause(TBool aPause)
    {
    if(iParser != NULL)
        {
        if(aPause)
            iDriver->setTimerCallback(NULL, NULL);
        else
            iDriver->setTimerCallback(iParser, MidiParser::timerCallback);
        }
    }

TInt CMidiPlayer::Volume()
    {
	return iVolume;
    }

void CMidiPlayer::Stop()
    {
     if(iParser!= NULL)
         iParser->unloadMusic();
    delete iParser;
    iParser = NULL;
    iDriver->setTimerCallback(NULL, NULL);
    }

CMidiPlayer::~CMidiPlayer()
    {
    if(iParser != NULL)
        iParser->unloadMusic();
    delete iParser;
    if(iDriver != NULL)
        iDriver->close();
    delete iDriver;
    delete iMidiDataBuffer;
    }

CAudioPlayer* g_AudioPlayer = NULL;

PremixProc *_premixProc = NULL;
void *_premixParam = NULL;

/**
* Set the premix procedure. This is mainly used for the adlib music, but
* is not limited to it. The premix proc is invoked by the mixer whenever
* it needs to generate any data, before any other mixing takes place. The
* premixer than has a chanve to fill the mix buffer with data (usually
* music samples). It should generate the specified number of 16bit stereo
* samples (i.e. len * 4 bytes). The endianess of these samples shall be
* the native endianess.
*/
void setupPremix(PremixProc *proc, void *param)
{
	_premixProc = proc;
	_premixParam = param;
}

extern "C"
{
	void SetMusicVolume(int aVolume)
	{
		g_AudioPlayer->Midi().SetVolume(aVolume*2);
	}

	int GetMusicVolume()
	{
		return g_AudioPlayer->Midi().Volume();
	}

	void CreateAudioPlayer()
	{
		g_AudioPlayer = new (ELeave) CAudioPlayer;
		g_AudioPlayer->ConstructL();
	}

	void StopAudioPlayer()
	{
		delete g_AudioPlayer;
		g_AudioPlayer = NULL;
	}

	void PlayMIDISong(const char* filename, int looping)
	{
		TFileName fname;
		TPtrC8 ptr((unsigned char*) filename, strlen(filename));
		fname.Copy(ptr);
		g_AudioPlayer->Midi().PlayMidiFromFileL(fname, looping);
	}

	void PlayMIDISongFromBuffer(const unsigned char* buffer,unsigned int midi_size, int looping)
	{
		TPtrC8 ptr(buffer, midi_size);
		HBufC8* newbuffer = ptr.AllocLC();
		g_AudioPlayer->Midi().PlayMidiL(newbuffer, looping);
		CleanupStack::Pop(newbuffer);
	}

	void StopMIDISong()
	{
		g_AudioPlayer->Midi().Stop();
	}

	bool IsMidiPlaying()
	{
		if(g_AudioPlayer != NULL)
		{
			return g_AudioPlayer->Midi().IsPlaying();
		}

		return false;
	}

	bool IsMidiLoaded()
	{
		if(g_AudioPlayer != NULL)
		{
			return g_AudioPlayer->Midi().IsLoaded();
		}

		return false;
	}


	void GenerateSamples(signed short int* buf, unsigned int len)
	{
		if(IsMidiLoaded() && _premixProc != NULL)
		{
			_premixProc(_premixParam, buf, len);
		}
	}



}
////////////////////////////////////////////////////////////////////////////////////////////

CAudioPlayer::CAudioPlayer() 
    {
    }


void CAudioPlayer::ConstructL()
    {

    iMidiPlayer = new (ELeave) CMidiPlayer(*iMixer);
    
    iMidiPlayer->ConstructL();

	SetMusicVolume(MaxVolume());
    }


CMidiPlayer& CAudioPlayer::Midi()
    {
    return *iMidiPlayer;
    }



CAudioPlayer::~CAudioPlayer()
    {
    delete iMidiPlayer;
    }


TInt CAudioPlayer::MaxVolume() const
    {
    return TInt(255);
    }


void warning(char const* aStr, ...)
    {
#ifdef WARNINGS_AND_DEBUGS
    VA_LIST list;
    VA_START(list, aStr);
    TBuf8<1024> data;
    const TPtrC8 txt((const TUint8*)aStr);
    data.FormatList(txt, list);
    TBuf<256> warn;
    warn.Copy(data);
    User::InfoPrint(warn);
    VA_END(list);
#else
    aStr;
#endif
    }

void debug(int val, char const* aStr, ...)
    {
#ifdef WARNINGS_AND_DEBUGS
    VA_LIST list;
    VA_START(list, aStr);
    TBuf8<1024> data;
    const TPtrC8 txt((const TUint8*)aStr);
    data.FormatList(txt, list);
    TBuf<256> warn;
    warn.Copy(data);
    warn.AppendFormat(_L(" %d"), val);
    User::InfoPrint(warn);
    VA_END(list);
#else
    val;
    aStr;
#endif
    }

void error(char const* aStr, ...)
    {
    VA_LIST list;
    VA_START(list, aStr);
    TBuf8<1024> data;
    const TPtrC8 txt((const TUint8*)aStr);
    data.FormatList(txt, list);
    TBuf<256> panic;
    panic.Copy(data);
   // User::Panic(panic , KErrGeneral);
    VA_END(list);
    }

